//+------------------------------------------------------------------+
//|                                                Complex_class.mqh |
//|                                                           denkir |
//|                                                 denkir@gmail.com |
//+------------------------------------------------------------------+
#property copyright "denkir"
#property link      "denkir@gmail.com"
//+------------------------------------------------------------------+
//|          Complex Class definition                                |
//+------------------------------------------------------------------+
class Complex
  {
public:
   double            re,im; //re - real component of a complex number, im - imaginary component
public:
   void              Complex(){};                                   // default constructor
   void              setComplex(double rE,double iM){re=rE; im=iM;};//set method (1-st variant)
   void              setComplex(double rE){re=rE; im=0;};           //set method (2-nd variant)
   void              ~Complex(){};                                  //destructor
   void              opEqual(const Complex &y){re=y.re;im=y.im;};   //operator=
   void              opPlus(const Complex &x,const Complex &y);     //operator+   
   void              opPlusEq(const Complex &y);                    //operator+=
   void              opMinus(const Complex &x,const Complex &y);    //operator- 
   void              opMult(const Complex &x,const Complex &y);     //operator*   
   void              opMultEq(const Complex &y);                    //operator*= (1-st variant)     
   void              opMultEq(const double y);                      //operator*= (2-nd variant)    
   void              conjugate(const Complex &y);                   //conjugation of complex numbers
   double            norm();                                        //normalization
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void setComplexArr(Complex &cArr[],double &rE[])
//returns an array of complex numbers from a unidimensional array of real numbers
  {
   int n=ArraySize(rE);
   for(int i=0;i<n;i++)
     {
      cArr[i].re=rE[i]; cArr[i].im=0;
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void setComplexArr2(Complex &cArr[],double &rA[][2])
//returns an array of complex numbers for a two-dimensional array of real numbers
  {
   int n=ArraySize(rA)/2;
   for(int i=0;i<n;i++)
     {
      cArr[i].re=rA[i][0];
      cArr[i].im=rA[i][1];;
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void getComplexArr(double &rE[][2],Complex &cArr[])
//returns a two-dimensional array of real numbers from an array of complex numbers
  {
   int n=ArraySize(cArr);
   for(int i=0;i<n;i++)
     {
      rE[i][0]=cArr[i].re;
      rE[i][1]=cArr[i].im;
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void Complex::opMinus(const Complex &x,const Complex &y) //operator-
  {
   re=x.re-y.re; 
   im= x.im - y.im;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void Complex::opMult(const Complex &x,const Complex &y) //operator*
  {
   re=x.re*y.re-x.im*y.im; //
   im= x.re*y.im + x.im*y.re;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void Complex::opMultEq(const Complex &y) //operator*= (1- )  
  {
   const double temp=re;
   re = re * y.re - im * y.im;
   im = im * y.re + temp*y.im;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void Complex::opMultEq(const double y)  //operator* (2- )  
  {
   re*=y;
   im*=y;
  }
//+------------------------------------------------------------------+
double Complex::norm() //
  {
   return re*re+im*im;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void Complex::opPlus(const Complex &x,const Complex &y) //operator+
  {
   re = x.re + y.re;
   im = x.im + y.im;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void Complex::opPlusEq(const Complex &y) //operator+=
  {
   re += y.re;
   im += y.im;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void Complex::conjugate(const Complex &y) //conjugation of complex numbers
  {
   re = y.re;
   im = (-1)*y.im;
  }
//+------------------------------------------------------------------+